#include <gtk/gtk.h>
#include <stdlib.h>
#include <pthread.h>
#include <iostream>
#include <strstream>
#include <fstream>

using namespace std;

#include "general.h"
#include "GObject.h"
#include "snmppoll.h"
#include "Window.h"
#include "SizeW.h"
#include "Console.h"
#include "DrawingArea.h"
#include "EventBox.h"
#include "AboutBox.h"
#include "Menu.h"
#include "Poll.h"
#include "Parser.h"
#include "ScrolledWindow.h"
#include "SNMPObject.h"
#include "PopDown.h"
#include "StatusBar.h"

pthread_mutex_t m_do_update = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t m_task_ended = PTHREAD_MUTEX_INITIALIZER;

struct sPthreadInfo
{
    Parser *parser;
    DrawingArea *drawingarea;
}arg;

Console *global_console;

#include "menu_elements.h"

//strange SNMP-info-urile din retea
void * repoll_thread(void *data)
{
	int ret;
	sPthreadInfo *p=(sPthreadInfo *)data;
	vector<SNMPObject *> &obj=(p->parser)->getObjects();
	while(true)
	{
		pthread_mutex_lock(&m_do_update);
		//begin A
		pthread_mutex_lock(&m_task_ended);
		//cout<<"------------"<<endl;
		for(int i=0; i<obj.size(); i++)
		{
		    obj[i]->update();
		}
		pthread_mutex_unlock(&m_task_ended);
		//end A
	}
}

//trezeste o data la X secunde thread-ul de redesenare
gint wake_up_poll_thread(gpointer data) //initiaza repoll
{
	int ret;
	sPthreadInfo *p=(sPthreadInfo *)data;
	
	if(pthread_mutex_trylock(&m_task_ended)==0) //nu e in A
	{
		pthread_mutex_unlock(&m_do_update);
		pthread_mutex_unlock(&m_task_ended);
	}
    return 1;
}

gint redraw_map(gpointer data)
{
	sPthreadInfo *p=(sPthreadInfo *)data;
	vector<SNMPObject *> &obj=(p->parser)->getObjects();
	DrawingArea *d=p->drawingarea;
	for(int i=0; i<obj.size(); i++)
	{
        obj[i]->draw(d);
    }
    d->updateScreen();
    
    return 1;
}		

struct click_callback_data
{
	StatusBar *_sb;
	Parser &_parser;
	PopDown &_pd;
	click_callback_data(StatusBar *sb, Parser &parser, PopDown &pd):
		_parser(parser), _pd(pd) {
		_sb = sb;
	};
};

gint click_callback(GtkWidget *widget, GdkEventButton *event, click_callback_data *data)
{
	if(event->button == 2)
	{
		strstream a;
		a<<event->x<<" "<<event->y<<ends;
		data->_sb->push( a.str() );
	}
	else if(event->button == 1)
	{
		//se vor da informatii despre obiectul click-uit
		//deci, se va verifica daca coordonatele apartin suprafetei vreunui obiect
		//si daca da, se va trimite in statusbar chestia asta.
		vector<SNMPObject *>& obj = (data->_parser).getObjects();
		for(int x=0; x<obj.size(); x++)
		{
			if(obj[x]->inRange(event->x, event->y))
			{
				data->_sb->push( obj[x]->getState() );
				break;
			}
		}
	}
	else
	{
		//am nevoie de pointer catre popdown
		//identific si setez target-ul
		//apelez un menu_show
		//cand userul alege o optiune, callback-ul identifica dupa widget optiunea aleasa
		//si gaseste targetul setat in structura respectiva, care mai contine si un
		//pointer catre popdown
		vector<SNMPObject *>& obj = (data->_parser).getObjects();
		for(int x=0; x<obj.size(); x++)
		{
			if(obj[x]->inRange(event->x, event->y))
			{
				data->_pd.setTarget(obj[x]);
				data->_pd.show(event);
				break;
			}
		}
		return TRUE;
	}
	return FALSE;
}

int main(int argc, char *argv[])
{
	_snmp_init_app("monitor");
	
	gtk_init(&argc,&argv);

	string conf;
	if(argc==1)
	{
	    cerr<<"Configuration file not specified. Using /etc/snmp.monrc"<<endl;
	    conf="/etc/snmp.monrc";
	}
	else if(argc==2)
	{
	    conf=argv[1];
	}
	else
	{
	    cerr<<"Use: snmp_monitor [configuration_file]"<<endl;
	}
	
	Parser parser(conf);

	Window window(parser.getWidth()+22,parser.getHeight()+65);
	DrawingArea drawingarea(parser.getWidth(), parser.getHeight(), "drawingarea");
	ScrolledWindow scrolledwindow("scrolledwindow");
	EventBox event_box("EventBox");
	Menu menu(mainmenu,sizeof(mainmenu),"menubar",&drawingarea);
	StatusBar statusbar;
	
	window.addWidget(&menu);
	window.addWidget(&drawingarea);
	window.addWidget(&scrolledwindow);	
	window.addWidget(&event_box);
	window.addWidget(&statusbar);

	menu.atachAccelerators(window.getWidget("toplevel"));
				
	gtk_signal_connect(GTK_OBJECT(window.getWidget("toplevel")), "destroy", GTK_SIGNAL_FUNC(gtk_exit), NULL);
	gtk_signal_connect(GTK_OBJECT(window.getWidget("toplevel")), "delete_event", GTK_SIGNAL_FUNC(gtk_exit), NULL);
	
	PopDown pd("popdown");
	pd.addItem("Telnet..", "xterm -e telnet");
	pd.addItem("Ping..", "xterm -e ping");

	click_callback_data _maincb(&statusbar, parser, pd);
	gtk_signal_connect(GTK_OBJECT(event_box.getWidget()), "button_press_event", GTK_SIGNAL_FUNC(click_callback), &_maincb);

	GtkWidget *vbox = window.getWidget("vbox");
	gtk_box_pack_start(GTK_BOX(vbox),menu.getWidget(),FALSE,TRUE,1);
	gtk_box_pack_start(GTK_BOX(vbox),scrolledwindow.getWidget(),TRUE,TRUE,1);
	event_box.setChild(drawingarea.getWidget());
	scrolledwindow.setChild(event_box.getWidget());
	gtk_box_pack_start(GTK_BOX(vbox),statusbar.getWidget(),FALSE,TRUE,1); 

	window.draw();
	
	if(parser.getBackground() != "")
	{
		drawingarea.setBackground((char *)parser.getBackground().c_str(),
			parser.getWidth(), parser.getHeight());
		drawingarea.updateScreen();
	}

	arg.parser=&parser;
	arg.drawingarea=&drawingarea;
	
	pthread_mutex_init(&m_do_update, NULL);
	pthread_mutex_init(&m_task_ended, NULL);

	pthread_t tid;
	pthread_create(&tid, NULL, repoll_thread, &arg);
	
	gtk_timeout_add(parser.getTimeout(), wake_up_poll_thread, &arg);	
	gtk_timeout_add(5000, redraw_map, &arg);	
	
	statusbar.push("SNMP Monitor started.");
	
	Console console;
	global_console = &console;
	console.addText("SNMP Monitor started\n", console._information);
	
	redraw_map(&arg);

	gtk_main();

	return 0;
}
